package cz.upol.fpj.lecture10

import scala.annotation.tailrec

object Parameters {
  
  // variadicka funkce
  def evens(ints: Int*) = {
    ints.filter(x => (x % 2) == 0)
  }
  
  // ukazka pojmenovanych atributu
  def speed(dist: Double, time: Double) =
    dist / time;
  
  // implicitni parametr
  def inc(n: Int, i: Int = 1) = n + i;
  
  // currying implementovany primocarym zpusobem
  def currySumTraditional =
    ((a: Int) => 
      ((b: Int) => 
        ((c: Int) => a + b + c)));
  
  // currying implementovany s vyuzitim syntaxe Scaly
  def currySum(a: Int)(b: Int)(c: Int) = a + b + c;
  
  // genericka funkce
  def applyIf[T](pred: T => Boolean, f: T => T, value: T): T =
    if (pred(value)) f(value)
    else value;
  
  def abs(n: Int)    = applyIf((_: Int) < 0, - (_: Int), n)
  def abs(n: Double) = applyIf[Double](_ < 0, - _, n)
  def mkImportant    = applyIf[String](_.endsWith("!"), _.toUpperCase(), _: String)
  
  // call-by-name
  def doIf(pred: Boolean, action: => Unit) =
    if (pred) action
    
  @tailrec
  def repeat(n: Int)(action: => Unit): Unit =
    if (n > 0) {
      action
      repeat(n - 1)(action)
    };
  
  
  
  def main(args: Array[String]): Unit = {
    evens(10, 11, 21, 20) foreach println
    println(speed(100, 2))
    println(speed(time = 2, dist = 100))
    println(inc(10))
    println(inc(10, 2))
    
    val sum2a = currySumTraditional(10)
    val sum2b = currySum(10) _
    println(sum2a(1)(2))
    println(sum2b(1)(2))
    println(mkImportant("abc"))
    
    doIf(false, println("aaa"))
    doIf(true, println("bbb"))
    
    repeat(5)(println("a"))
    
    repeat(5) { 
      println("b")
    }
  }
}